Imports Microsoft.VisualBasic
Imports System
Imports System.Data
Imports System.Configuration
Imports System.Data.SqlClient
Imports System.Collections.Generic
Imports System.Collections.Specialized
Imports System.Web.UI.WebControls
Imports System.IO
Imports System.Web.UI
Imports System.Web
Imports Core35.DAL.Helpers



' CUSTOMERS 
' A class to hold customers data and a static class for the behavior according to the 
' Table Data Gateway pattern

Namespace Core35.DAL
	' CUSTOMER 
	' Holds information about the customer
	Public Class Customer
		Private _companyname, _contactname, _contacttitle, _id As String
		Private _street, _postalcode, _city, _country, _phone, _region, _fax As String

		Public Sub New()
		End Sub

		#Region "PROPERTIES"

		' ID
		Public Property ID() As String
			Get
				Return _id
			End Get
			Set(ByVal value As String)
				_id = value
			End Set
		End Property

		' CompanyName
		Public Property CompanyName() As String
			Get
				Return _companyname
			End Get
			Set(ByVal value As String)
				_companyname = value
			End Set
		End Property

		' ContactName
		Public Property ContactName() As String
			Get
				Return _contactname
			End Get
			Set(ByVal value As String)
				_contactname = value
			End Set
		End Property

		' ContactTitle
		Public Property ContactTitle() As String
			Get
				Return _contacttitle
			End Get
			Set(ByVal value As String)
				_contacttitle = value
			End Set
		End Property

		' Street
		Public Property Street() As String
			Get
				Return _street
			End Get
			Set(ByVal value As String)
				_street = value
			End Set
		End Property

		' PostalCode
		Public Property PostalCode() As String
			Get
				Return _postalcode
			End Get
			Set(ByVal value As String)
				_postalcode = value
			End Set
		End Property

		' City
		Public Property City() As String
			Get
				Return _city
			End Get
			Set(ByVal value As String)
				_city = value
			End Set
		End Property

		' Country
		Public Property Country() As String
			Get
				Return _country
			End Get
			Set(ByVal value As String)
				_country = value
			End Set
		End Property

		' Phone
		Public Property Phone() As String
			Get
				Return _phone
			End Get
			Set(ByVal value As String)
				_phone = value
			End Set
		End Property

		' Fax
		Public Property Fax() As String
			Get
				Return _fax
			End Get
			Set(ByVal value As String)
				_fax = value
			End Set
		End Property

		' Region
		Public Property Region() As String
			Get
				Return _region
			End Get
			Set(ByVal value As String)
				_region = value
			End Set
		End Property

		#End Region


		#Region "METHODS"

		' ToString
		Public Overrides Function ToString() As String
			Return String.Format("{0} [{1}]", _companyname, _id)
		End Function
		#End Region
	End Class


	Public NotInheritable Class NoCustomer
		Inherits Customer
		Public Shared ReadOnly Instance As New NoCustomer()

		Public ReadOnly Property Tag() As String
			Get
				Return "Null Customer"
			End Get
		End Property
	End Class

	' CLASS InvalidCustomer: represents an invalid customer instance
	Public NotInheritable Class InvalidCustomer
		Inherits Customer
		Public Shared ReadOnly Instance As New InvalidCustomer()

		Public ReadOnly Property Tag() As String
			Get
				Return "Invalid Customer"
			End Get
		End Property
	End Class

	' CUSTOMERCOLLECTION
	' Collection of customer objects
	Public Class CustomerCollection
		Inherits List(Of Customer)
	End Class


	' CUSTOMERS--customer gateway
	' Behavior of the customer
	Public Class Customers
		Public Shared ReadOnly Property ConnectionString() As String
			Get
				Return ConfigurationManager.ConnectionStrings("NorthwindConnectionString").ConnectionString
			End Get
		End Property

		#Region "METHOD: Load"
		' Load
		Public Shared Function Load(ByVal id As String) As Customer
			If String.IsNullOrEmpty(id) Then
				Return Nothing ' new NoCustomer();
			End If

			Using conn As New SqlConnection(ConnectionString)
				Dim cmd As New SqlCommand(CustomerCommands.cmdLoad, conn)
				cmd.Parameters.AddWithValue("@id", id)

				conn.Open()
				Dim reader As SqlDataReader = cmd.ExecuteReader()
				reader.Read()
				Dim cust As Customer = HelperMethods.GetCustomer(reader)
				reader.Close()
				conn.Close()

				Return cust
			End Using
		End Function
		#End Region


		#Region "METHOD: LoadSet"
		' Load
		Public Shared Function LoadSet(ByVal match As String) As CustomerCollection
			If String.IsNullOrEmpty(match) Then
				Return LoadAll()
			End If

			Dim coll As New CustomerCollection()

			Using conn As New SqlConnection(ConnectionString)
				Dim cmd As New SqlCommand(CustomerCommands.cmdLoadSet, conn)
				cmd.Parameters.AddWithValue("@id", match)

				conn.Open()
				Dim reader As SqlDataReader = cmd.ExecuteReader()
				HelperMethods.FillCustomerList(coll, reader)
				reader.Close()
				conn.Close()

				Return coll
			End Using
		End Function
		#End Region


		#Region "METHOD: LoadAll"
		' LoadAll
		Public Shared Function LoadAll() As CustomerCollection
			Return LoadAll(-1, 0)
		End Function
		Public Shared Function LoadAll(ByVal totalRows As Integer, ByVal firstRow As Integer) As CustomerCollection
			Return LoadAll(totalRows, firstRow, "")
		End Function
		Public Shared Function LoadAll(ByVal totalRows As Integer, ByVal firstRow As Integer, ByVal sortExpression As String) As CustomerCollection
			Dim coll As New CustomerCollection()

			Using conn As New SqlConnection(ConnectionString)
				Dim cmd As New SqlCommand(CustomerCommands.cmdLoadAll, conn)
				If (Not String.IsNullOrEmpty(sortExpression)) Then
					cmd.CommandText &= " ORDER BY " & sortExpression
				End If

				conn.Open()
				Dim reader As SqlDataReader = cmd.ExecuteReader()
				HelperMethods.FillCustomerList(coll, reader, totalRows, firstRow)
				reader.Close()
				conn.Close()
			End Using

			Return coll
		End Function
		#End Region


		#Region "METHOD: LoadByCountry"
		' LoadByCountry
		Public Shared Function LoadByCountry(ByVal country As String) As CustomerCollection
			Return LoadByCountry(country, -1, 0, "")
		End Function
		Public Shared Function LoadByCountry(ByVal country As String, ByVal totalRows As Integer, ByVal firstRow As Integer) As CustomerCollection
			Return LoadByCountry(country, totalRows, firstRow, "")
		End Function
		Public Shared Function LoadByCountry(ByVal country As String, ByVal totalRows As Integer, ByVal firstRow As Integer, ByVal sortExpression As String) As CustomerCollection
			Dim coll As New CustomerCollection()

			Using conn As New SqlConnection(ConnectionString)
				Dim cmd As New SqlCommand(CustomerCommands.cmdLoadByCountry, conn)
				cmd.Parameters.AddWithValue("@country", country)
				If (Not String.IsNullOrEmpty(sortExpression)) Then
					cmd.CommandText &= " ORDER BY " & sortExpression
				End If

				conn.Open()
				Dim reader As SqlDataReader = cmd.ExecuteReader()
				HelperMethods.FillCustomerList(coll, reader, totalRows, firstRow)
				reader.Close()
				conn.Close()
			End Using

			Return coll
		End Function
		#End Region


		#Region "METHOD: GetCustomerCountries"
		' GetCustomerCountries
		Public Shared Function GetCustomerCountries() As StringCollection
			Dim countries As New StringCollection()

			Using conn As New SqlConnection(ConnectionString)
				Dim cmd As New SqlCommand(CustomerCommands.cmdCustCountries, conn)

				conn.Open()
				Dim reader As SqlDataReader = cmd.ExecuteReader()
				HelperMethods.FillCountryList(countries, reader)
				reader.Close()
				conn.Close()
			End Using

			Return countries
		End Function
		#End Region


		#Region "METHOD: Save"
		' Save 
		Public Shared Function Save(ByVal cust As Customer) As Customer
			If cust Is Nothing Then
				Return InvalidCustomer.Instance
			End If

			Using conn As New SqlConnection(ConnectionString)
				Dim cmd As New SqlCommand(CustomerCommands.cmdSave, conn)
				cmd.Parameters.AddWithValue("@id", cust.ID)
				cmd.Parameters.AddWithValue("@companyname", cust.CompanyName)
				cmd.Parameters.AddWithValue("@city", cust.City)
				cmd.Parameters.AddWithValue("@address", cust.Street)
				cmd.Parameters.AddWithValue("@country", cust.Country)
'                cmd.Parameters.AddWithValue("@contactname", cust.ContactName);
'                cmd.Parameters.AddWithValue("@phone", cust.Phone);
'                cmd.Parameters.AddWithValue("@postalcode", cust.PostalCode);
'                cmd.Parameters.AddWithValue("@region", cust.Region);
'                cmd.Parameters.AddWithValue("@contacttitle", cust.ContactTitle);
'                cmd.Parameters.AddWithValue("@fax", cust.Fax);

				conn.Open()
				cmd.ExecuteNonQuery()
				conn.Close()
			End Using

			Return cust
		End Function

		#End Region


		#Region "METHOD: Add"
		' Add 
		Public Shared Sub Add(ByVal cust As Customer)
			Using conn As New SqlConnection(ConnectionString)
				Dim cmd As New SqlCommand(CustomerCommands.cmdInsert, conn)
				cmd.Parameters.AddWithValue("@id", cust.ID)
				cmd.Parameters.AddWithValue("@companyname", cust.CompanyName)
				cmd.Parameters.AddWithValue("@city", cust.City)
				cmd.Parameters.AddWithValue("@address", cust.Street)
				cmd.Parameters.AddWithValue("@country", cust.Country)
				'                cmd.Parameters.AddWithValue("@phone", cust.Phone);
				'                cmd.Parameters.AddWithValue("@postalcode", cust.PostalCode);
				'                cmd.Parameters.AddWithValue("@region", cust.Region);
				'                cmd.Parameters.AddWithValue("@contactname", cust.ContactName);
				'                cmd.Parameters.AddWithValue("@contacttitle", cust.ContactTitle);
				'                cmd.Parameters.AddWithValue("@fax", cust.Fax);

				conn.Open()
				cmd.ExecuteNonQuery()
				conn.Close()
				Return
			End Using
		End Sub

		#End Region


		#Region "METHOD: Delete"
		' Delete 
		Public Shared Sub Delete(ByVal id As String)
			Using conn As New SqlConnection(ConnectionString)
				Dim cmd As New SqlCommand(CustomerCommands.cmdDelete, conn)
				cmd.Parameters.AddWithValue("@id", id)

				conn.Open()
				cmd.ExecuteNonQuery()
				conn.Close()
				Return
			End Using
		End Sub
		Public Shared Sub Delete(ByVal cust As Customer)
			Delete(cust.ID)
		End Sub
		#End Region


		#Region "METHOD: FindOrdersByCustomerAsMarkup"
		Public Shared Function FindOrdersByCustomerAsMarkup(ByVal id As String) As String
			Using conn As New SqlConnection(ConnectionString)
				Dim cmd As New SqlCommand(CustomerCommands.cmdOrdersAll, conn)
				cmd.Parameters.AddWithValue("@id", id)

				conn.Open()
				Dim reader As SqlDataReader = cmd.ExecuteReader()
				reader.Read()
				Dim markup As String = HelperMethods.ComposeOrdersAsGrid(reader)
				reader.Close()
				conn.Close()

				Return markup
			End Using
		End Function
		#End Region 

	End Class


	' Helpers
	Public Partial NotInheritable Class HelperMethods
		' GetCustomer
        Public Shared Function GetCustomer(ByVal reader As SqlDataReader) As Customer
            Dim cust As New Customer()
            If reader.IsClosed Then
                reader.Read()
            End If

            cust.ID = reader("customerid").ToString()
            cust.CompanyName = reader("companyname").ToString()
            cust.ContactName = reader("contactname").ToString()
            cust.ContactTitle = reader("contacttitle").ToString()

            cust.City = reader("city").ToString()
            cust.Street = reader("address").ToString()
            cust.Country = reader("country").ToString()

            cust.Region = reader("region").ToString()
            cust.PostalCode = reader("postalcode").ToString()
            cust.Phone = reader("phone").ToString()

            cust.Fax = reader("fax").ToString()

            Return cust
        End Function

		' FillCustomerList
		Public Shared Sub FillCustomerList(ByVal coll As CustomerCollection, ByVal reader As SqlDataReader)
			FillCustomerList(coll, reader, -1, 0)
		End Sub
		Public Shared Sub FillCustomerList(ByVal coll As CustomerCollection, ByVal reader As SqlDataReader, ByVal totalRows As Integer, ByVal firstRow As Integer)
			Dim index As Integer = 0
			Dim readMore As Boolean = True

			Do While reader.Read()
				If index >= firstRow AndAlso readMore Then
					If coll.Count >= totalRows AndAlso totalRows >0 Then
						readMore = False
					Else
						Dim cust As Customer = HelperMethods.GetCustomer(reader)
						coll.Add(cust)
					End If
				End If
				index += 1
			Loop
		End Sub
		Public Shared Function ComposeOrdersAsGrid(ByVal reader As SqlDataReader) As String
			Dim grid As New DataGrid()
			grid.DataSource = reader
			grid.DataBind()

			Dim writer As New StringWriter()
			Dim html As New HtmlTextWriter(writer)
			grid.RenderControl(html)
			Return writer.ToString()
		End Function
	End Class
End Namespace
