Imports Microsoft.VisualBasic
Imports System
Imports System.Data
Imports System.Configuration
Imports System.Data.SqlClient
Imports System.Collections.Generic
Imports System.Collections.Specialized
Imports Core35.DAL.Helpers



' EMPLOYEE 
' A class to hold employee data and a static class for the behavior according to the 
' Table Data Gateway pattern

Namespace Core35.DAL
	' EMPLOYEE 
	' Holds information about the employee
	Public Class Employee
		Private _firstname, _lastname, _title, _titleofcourtesy, _notes, _extension As String
		Private _street, _postalcode, _city, _country, _phone, _region, _fax As String
		Private _id, _reportsTo As Integer
		Private _birthdate, _hiredate As DateTime

		Public Sub New()
		End Sub

		#Region "PROPERTIES"

		' ID
		Public Property ID() As Integer
			Get
				Return _id
			End Get
			Set(ByVal value As Integer)
				_id = value
			End Set
		End Property

		' FirstName
		Public Property FirstName() As String
			Get
				Return _firstname
			End Get
			Set(ByVal value As String)
				_firstname = value
			End Set
		End Property

		' LastName
		Public Property LastName() As String
			Get
				Return _lastname
			End Get
			Set(ByVal value As String)
				_lastname = value
			End Set
		End Property

		' Title
		Public Property Title() As String
			Get
				Return _title
			End Get
			Set(ByVal value As String)
				_title = value
			End Set
		End Property

		' TitleOfCourtesy
		Public Property TitleOfCourtesy() As String
			Get
				Return _titleofcourtesy
			End Get
			Set(ByVal value As String)
				_titleofcourtesy = value
			End Set
		End Property

		' BirthDate
		Public Property BirthDate() As DateTime
			Get
				Return _birthdate
			End Get
			Set(ByVal value As DateTime)
				_birthdate = value
			End Set
		End Property

		' HireDate
		Public Property HireDate() As DateTime
			Get
				Return _hiredate
			End Get
			Set(ByVal value As DateTime)
				_hiredate = value
			End Set
		End Property

		' ReportsTo
		Public Property ReportsTo() As Integer
			Get
				Return _reportsTo
			End Get
			Set(ByVal value As Integer)
				_reportsTo = value
			End Set
		End Property

		' Extension
		Public Property Extension() As String
			Get
				Return _extension
			End Get
			Set(ByVal value As String)
				_extension = value
			End Set
		End Property

		' Notes
		Public Property Notes() As String
			Get
				Return _notes
			End Get
			Set(ByVal value As String)
				_notes = value
			End Set
		End Property

		Public Property Street() As String
			Get
				Return _street
			End Get
			Set(ByVal value As String)
				_street = value
			End Set
		End Property

		' PostalCode
		Public Property PostalCode() As String
			Get
				Return _postalcode
			End Get
			Set(ByVal value As String)
				_postalcode = value
			End Set
		End Property

		' City
		Public Property City() As String
			Get
				Return _city
			End Get
			Set(ByVal value As String)
				_city = value
			End Set
		End Property

		' Country
		Public Property Country() As String
			Get
				Return _country
			End Get
			Set(ByVal value As String)
				_country = value
			End Set
		End Property

		' Phone
		Public Property Phone() As String
			Get
				Return _phone
			End Get
			Set(ByVal value As String)
				_phone = value
			End Set
		End Property

		' Fax
		Public Property Fax() As String
			Get
				Return _fax
			End Get
			Set(ByVal value As String)
				_fax = value
			End Set
		End Property

		' Region
		Public Property Region() As String
			Get
				Return _region
			End Get
			Set(ByVal value As String)
				_region = value
			End Set
		End Property
		#End Region


		#Region "METHODS"

		' ToString
		Public Overrides Function ToString() As String
			Return String.Format("{0} {1}", _firstname, _lastname)
		End Function
		#End Region
	End Class


	' NULLEMPLOYEE
	Public Class NullEmployee
		Inherits Employee
		Public Sub New()
			ID = -1
		End Sub
	End Class


	' EMPLOYEECOLLECTION
	' Collection of employee objects
	Public Class EmployeeCollection
		Inherits List(Of Employee)
	End Class


	' EMPLOYEES--employee gateway
	' Behavior of the employee
	Public Class Employees
		Public Shared ReadOnly Property ConnectionString() As String
			Get
				Return ConfigurationManager.ConnectionStrings("NorthwindConnectionString").ConnectionString
			End Get
		End Property

		#Region "METHOD: Load"
		' Load
		Public Shared Function Load(ByVal id As Integer) As Employee
			Using conn As New SqlConnection(ConnectionString)
				Dim cmd As New SqlCommand(EmployeeCommands.cmdLoad, conn)
				cmd.Parameters.AddWithValue("@id", id)

				conn.Open()
				Dim reader As SqlDataReader = cmd.ExecuteReader()
				reader.Read()
				Dim emp As Employee = HelperMethods.GetEmployee(reader)
				reader.Close()
				conn.Close()

				Return emp
			End Using
		End Function
		#End Region


		#Region "METHOD: LoadAll"
		' LoadAll
		Public Shared Function LoadAll() As EmployeeCollection
			Dim coll As New EmployeeCollection()

			Using conn As New SqlConnection(ConnectionString)
				Dim cmd As New SqlCommand(EmployeeCommands.cmdLoadAll, conn)

				conn.Open()
				Dim reader As SqlDataReader = cmd.ExecuteReader()
				HelperMethods.FillEmployeeList(coll, reader)
				reader.Close()
				conn.Close()
			End Using

			Return coll
		End Function
		#End Region


		#Region "METHOD: LoadByCountry"
		' LoadAll
		Public Shared Function LoadByCountry(ByVal country As String) As EmployeeCollection
			Dim coll As New EmployeeCollection()

			Using conn As New SqlConnection(ConnectionString)
				Dim cmd As New SqlCommand(EmployeeCommands.cmdLoadByCountry, conn)
				cmd.Parameters.AddWithValue("@country", country)

				conn.Open()
				Dim reader As SqlDataReader = cmd.ExecuteReader()
				HelperMethods.FillEmployeeList(coll, reader)
				reader.Close()
				conn.Close()
			End Using

			Return coll
		End Function
		#End Region


		#Region "METHOD: GetEmployeeCountries"
		' GetEmployeeCountries
		Public Shared Function GetEmployeeCountries() As StringCollection
			Dim countries As New StringCollection()

			Using conn As New SqlConnection(ConnectionString)
				Dim cmd As New SqlCommand(EmployeeCommands.cmdEmpCountries, conn)

				conn.Open()
				Dim reader As SqlDataReader = cmd.ExecuteReader()
				HelperMethods.FillCountryList(countries, reader)
				reader.Close()
				conn.Close()
			End Using

			Return countries
		End Function
		#End Region


		#Region "METHOD: Save"
		' Save 
		Public Shared Sub Save(ByVal emp As Employee)
			Using conn As New SqlConnection(ConnectionString)
				Dim cmd As New SqlCommand(EmployeeCommands.cmdSave, conn)
				cmd.Parameters.AddWithValue("@id", emp.ID)
				cmd.Parameters.AddWithValue("@lastname", emp.LastName)
				cmd.Parameters.AddWithValue("@firstname", emp.FirstName)
				cmd.Parameters.AddWithValue("@birthdate", emp.BirthDate)
				cmd.Parameters.AddWithValue("@title", emp.Title)
				cmd.Parameters.AddWithValue("@country", emp.Country)

'
' *              In production code, you typically have stored procedures instead of SQL 
' *              commands. In a stored procedure, if there are situations in which only a
' *              subset of parameters can be passed, you first read the row, then assign
' *              default values to missing parameters and proceed with the update
' 

'
'				cmd.Parameters.AddWithValue("@titleofcourtesy", emp.TitleOfCourtesy);
'				cmd.Parameters.AddWithValue("@reportsto", emp.ReportsTo);
'				cmd.Parameters.AddWithValue("@notes", emp.Notes);
'				cmd.Parameters.AddWithValue("@hiredate", emp.HireDate);
'				cmd.Parameters.AddWithValue("@extension", emp.Extension);
'				cmd.Parameters.AddWithValue("@city", emp.City);
'				cmd.Parameters.AddWithValue("@phone", emp.Phone);
'				cmd.Parameters.AddWithValue("@postalcode", emp.PostalCode);
'				cmd.Parameters.AddWithValue("@region", emp.Region);
'				cmd.Parameters.AddWithValue("@address", emp.Street);
'
				conn.Open()
				cmd.ExecuteNonQuery()
				conn.Close()
				Return
			End Using
		End Sub

		#End Region

		' Delete


	End Class


	' Helpers
	Public Partial NotInheritable Class HelperMethods
		' GetEmployee
		Private Sub New()
		End Sub
		Public Shared Function GetEmployee(ByVal reader As SqlDataReader) As Employee
			Dim index As Integer
			If (Not reader.HasRows) Then
				Return Nothing
			End If

			Dim emp As New Employee()
			emp.ID = CInt(Fix(reader("employeeid")))
			emp.LastName = reader("lastname").ToString()
			emp.FirstName = reader("firstname").ToString()

			index = reader.GetOrdinal("birthdate")
			If reader.IsDBNull(index) Then
				emp.BirthDate = DateTime.MinValue
			Else
				emp.BirthDate = CDate(reader("birthdate"))
			End If

			index = reader.GetOrdinal("hiredate")
			If reader.IsDBNull(index) Then
				emp.HireDate = DateTime.MinValue
			Else
				emp.HireDate = CDate(reader("hiredate"))
			End If

			emp.Extension = reader("extension").ToString()
			emp.Title = reader("title").ToString()
			emp.TitleOfCourtesy = reader("titleofcourtesy").ToString()
			emp.Notes = reader("notes").ToString()

			index = reader.GetOrdinal("reportsto")
			If reader.IsDBNull(index) Then
				emp.ReportsTo = -1
			Else
				emp.ReportsTo = CInt(Fix(reader("reportsto")))
			End If
			emp.Street = reader("address").ToString()
			emp.City = reader("city").ToString()
			emp.Country = reader("country").ToString()
			emp.Region = reader("region").ToString()
			emp.PostalCode = reader("postalcode").ToString()
			emp.Phone = reader("homephone").ToString()
			emp.Fax = ""

			Return emp
		End Function

		' FillEmployeeList
		Public Shared Sub FillEmployeeList(ByVal coll As EmployeeCollection, ByVal reader As SqlDataReader)
			Do While reader.Read()
				Dim emp As Employee = HelperMethods.GetEmployee(reader)
				coll.Add(emp)
			Loop
		End Sub

		' FillCountryList
		Public Shared Sub FillCountryList(ByVal coll As StringCollection, ByVal reader As SqlDataReader)
			Do While reader.Read()
				If reader("country") IsNot Nothing Then
					Dim country As String = reader("country").ToString()
					If (Not String.IsNullOrEmpty(country)) Then
						coll.Add(country)
					End If
				End If
			Loop
		End Sub

	End Class
End Namespace
