Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Net
Imports System.IO
Imports System.ComponentModel
Imports System.Configuration



Namespace Samples.Services.FinanceInfo
	Public Class BaseFinanceInfoProvider
		Implements IFinanceInfoProvider

		''' <summary>
		''' Class constructor
		''' </summary>
		Public Sub New()
		End Sub


		#Region "IFinanceInfoProvider"

		''' <summary>
		''' IFinanceInfoProvider.GetQuotes(symbols)
		''' </summary>
		''' <param name="symbols">Comma-separated list of stock symbols to retrieve</param>
		''' <returns>Array of StockInfo</returns>
		Public Overridable Function GetQuotes(ByVal symbols As String) As StockInfo() Implements IFinanceInfoProvider.GetQuotes
			' Get the Finder component from config file (or use the default one)
			Dim finder As IFinanceInfoFinder = ResolveFinder()
			If finder Is Nothing Then
				Throw New NullReferenceException("Invalid Finder component.")
			End If

			Return finder.FindQuoteInfo(symbols)
		End Function


		''' <summary>
		''' IFinanceInfoProvider.GetQuotesAsHtml(symbols)
		''' </summary>
		''' <param name="symbols">Comma-separated list of stock symbols to retrieve</param>
		''' <returns>HTML message with response</returns>
		Public Overridable Function GetQuotesAsHtml(ByVal symbols As String) As String Implements IFinanceInfoProvider.GetQuotesAsHtml
			' Get quotes first
			Dim stocks() As StockInfo = GetQuotes(symbols)

			' Get the Renderer component from config file (or use the default one)
			Dim renderer As IFinanceInfoRenderer = ResolveRenderer()
			If renderer Is Nothing Then
				Throw New NullReferenceException("Invalid Renderer component.")
			End If

			Return renderer.GenerateHtml(stocks)
		End Function

		#End Region


		#Region "Resolvers"

		''' <summary>
		''' Reads the name of the finder component from config (or goes for a default class)
		''' </summary>
		''' <returns>Instance of a class that implements IFinanceInfoFinder</returns>
		Protected Overridable Function ResolveFinder() As IFinanceInfoFinder
			Return New RandomFinanceInfoFinder()
		End Function


		''' <summary>
		''' Reads the name of the renderer component from config (or goes for a default class)
		''' </summary>
		''' <returns>Instance of a class that implements IFinanceInfoRenderer</returns>
		Protected Overridable Function ResolveRenderer() As IFinanceInfoRenderer
			Dim className As String = ConfigurationManager.AppSettings(FinanceInfoUtils.CONFIG_RENDERER)
			If String.IsNullOrEmpty(className) Then
				Return New DefaultFinanceInfoRenderer()
			Else
				' NB: assuming the assembly is loaded!!! 
				' Use a different overload of CreateInstance for a more general scenario (class+assembly in a string)
				Dim t As Type = Type.GetType(className)
				Return TryCast(Activator.CreateInstance(t), IFinanceInfoRenderer)
			End If
		End Function

		#End Region

	End Class
End Namespace