Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.IO
Imports System.Net


Namespace Samples.Services.FinanceInfo
	Public Class YahooFinanceInfoFinder
		Inherits BaseFinanceInfoFinder
		#Region "Local variables"
		Private Const UrlBase As String = "http://download.finance.yahoo.com/d/quotes.csv?s={0}&f=sl1d1t1c1ohgvj1pp2owern&d=t"
		#End Region


		''' <summary>
		''' Class constructor
		''' </summary>
		Public Sub New()
		End Sub


		#Region "Virtuals"

		''' <summary>
		''' Indicates the name of the financial info provider
		''' </summary>
		Public Overrides ReadOnly Property ProviderName() As String
			Get
				Return "Finance Yahoo"
			End Get
		End Property


		''' <summary>
		''' Gets quotes and related change using the Finance Yahoo web service
		''' </summary>
		''' <param name="symbols">Comma-separated string indicating symbols to retrieve</param>
		''' <returns>Array of StockInfo objects</returns>
		Protected Overrides Function FindQuoteInfo(ByVal symbols As String) As StockInfo()
			' Set the URL to invoke
			Dim url As String = String.Format(UrlBase, symbols)

			' Prepare to work
			Dim output As String = String.Empty
			Dim reader As StreamReader = Nothing

			' Connect and get response
			Dim request As WebRequest = WebRequest.Create(url)
			Dim response As WebResponse = request.GetResponse()

			' Read the response
			reader = New StreamReader(response.GetResponseStream())
			Using reader
				output = reader.ReadToEnd()
				reader.Close()
			End Using

			' Parse and return
			Return Me.ParseResponseToArray(output, Me.ProviderName)
		End Function

		#End Region


		#Region "Helpers"

		''' <summary>
		''' Parse the response from Finance Yahoo to an array of custom objects
		''' </summary>
		''' <param name="response">Raw response as received from Yahoo</param>
		''' <param name="providerName">Name to store as the data provider</param>
		''' <returns>Array of StockInfo objects</returns>
		Private Function ParseResponseToArray(ByVal response As String, ByVal providerName As String) As StockInfo()
			' Prepare the container
			Dim data As List(Of StockInfo) = New List(Of StockInfo)()

			' Split response into rows
			Dim rows() As String = response.Split(New String() { Constants.vbLf }, StringSplitOptions.RemoveEmptyEntries)

			For i As Integer = 0 To rows.Length - 1
				Dim tokens() As String = rows(i).Split(","c)

				' Extract symbol information
				Dim stock As New StockInfo()

				stock.Symbol = RemoveQuotes(tokens(16))
				stock.Quote = tokens(1)
				stock.Day = RemoveQuotes(tokens(2))
				stock.Time = RemoveQuotes(tokens(3))
				stock.Change = RemoveQuotes(tokens(11))
				stock.ProviderName = providerName

				data.Add(stock)
			Next i

			Return data.ToArray()
		End Function


		''' <summary>
		''' Remove leading and trailing double quotes from a text string 
		''' </summary>
		''' <param name="originalText">Text to process</param>
		''' <returns>String without delimiting quotes</returns>
		Private Function RemoveQuotes(ByVal originalText As String) As String
			Dim temp As String = originalText.Trim()
			Dim newText As String = temp.Substring(1, temp.Length - 2)
			Return newText
		End Function
		#End Region
	End Class
End Namespace